mod spi_sd;

use alloc::sync::Arc;
use spin::Lazy;
use crate::bcache::*;

pub trait BlockDevice : Send + Sync {
  fn read_block(&self, block_id: usize, buf: &mut [u8]);
  fn write_block(&self, block_id: usize, buf: &[u8]);
}

static BLOCK_DEVICE: Lazy<spi_sd::SDCardWrapper> = Lazy::new(|| {
  spi_sd::SDCardWrapper::new()
});

struct BlockDevWrapper;

impl BlockIOWrapper for BlockDevWrapper {
  fn read_block(&self, blk: usize, buf: &mut[u8]) {
      BLOCK_DEVICE.read_block(blk, buf);
  }

  fn write_block(&self, blk: usize, buf: &[u8]) {
      BLOCK_DEVICE.write_block(blk, buf);
  }
}

impl BlockDevWrapper {
  pub fn new() -> Self {
    BlockDevWrapper { }
  }
}

static CACHE_BLK_DEVICE: Lazy<Arc<LRUCache>> = Lazy::new(|| {
  Arc::new(LRUCache::new(9, 8192, BlockDevWrapper::new()))
});

pub fn read_block(blk: usize, buf: &mut[u8]) {
  CACHE_BLK_DEVICE.read_block(blk, buf);
}

pub fn write_block(blk: usize, buf: &[u8]) {
  CACHE_BLK_DEVICE.write_block(blk, buf);
}
