pub mod address;
pub mod kalloc;
pub mod memory_space;
pub mod pgtbl;
pub mod pte_sv39;

use crate::config::*;
use crate::link_syms;
use crate::process::cpu::*;
use core::ops::Range;

pub use address::*;
pub use address::*;
pub use kalloc::*;
pub use memory_space::*;
pub use pgtbl::*;
pub use pte_sv39::*;

pub fn init() {
    kinit_page(frames_range());
}

pub fn activate_vm(pgtbl: &Pgtbl) {
    log!("vm":>"hart {} trying VM", init_hartid());
    // ################### TEST ######################
    use core::arch::asm;
    use riscv::register::satp;
    unsafe {
        satp::set(satp::Mode::Sv39, 0, pgtbl.root.0);
        asm!("sfence.vma");
    }
    log!("vm":>"Activated vm");
}

pub fn kernel_range() -> Range<PageNum> {
    let start = VirtualAddr(link_syms::skernel as usize).floor();
    let end = VirtualAddr(link_syms::frames as usize).floor();
    start..end
}

pub fn frames_range() -> Range<PageNum> {
    let start = VirtualAddr(link_syms::frames as usize).floor();
    let end = VirtualAddr(PHYS_FRAME_END).ceil();
    start..end
}
