use super::*;
use alloc::sync::Arc;

pub fn init_devfs() -> Result<(), FileErr> {
    let dev = match super::ROOT.create("dev", FileMode::empty(), InodeType::Directory) {
        Ok(dev) => {
            dev
        }
        Err(FileErr::InodeChildExist) => {
            super::ROOT.get_child("dev").unwrap()
        }
        Err(e) => {
            panic!("{:?}", e)
        }
    };
    assert!(super::ROOT.get_child("dev").is_ok());
    mount_at(Arc::new(DEVFS_ROOT), dev.ino().unwrap())
}

struct DEVFS_ROOT;

impl _Inode for DEVFS_ROOT {
    fn len(&self) -> usize {
        0
    }

    fn get_child(&self, name: &str) -> Result<Inode, FileErr> {
        if name == "vda2" {
            return Ok(Arc::new(DEVFS_VDA2))
        } else {
            Err(FileErr::InodeNotChild)
        }
    }
}

struct DEVFS_VDA2;

impl _Inode for DEVFS_VDA2 {
    fn len(&self) -> usize {
        0
    }
}
