#![no_std]
#![no_main]

mod syscall;
mod runtime;
mod console;

use syscall::*;
fn run_app(name: &str) {
    // println!("Runing {}", name);
    let fork_ret = syscall_fork();
    if fork_ret > 0 {
        let mut wstatus = 0;
        let mut rusage = 0;
        syscall_wait4(-1, &mut wstatus, 0, &mut rusage);
    } else if fork_ret == 0 {
        let mut argv: [usize; 2] = [0; 2];
        let envp: [usize; 1] = [0];
        let mut n: [u8; 512] = [0; 512];
        n[..name.len()].copy_from_slice(name.as_bytes());
        argv[0] = n.as_ptr() as usize;
        syscall_execve(unsafe {
            core::str::from_utf8_unchecked(&n[..name.len() + 1])
        }, &argv, &envp);
        syscall_exit(-1);
    } else {
        println!("Run: {} fork error", name)
    }
}

fn main() {
    run_app("argv");
    run_app("basename");
    run_app("clocale_mbfuncs");
    run_app("clock_gettime");
    run_app("crypt");
    run_app("dirname");
    run_app("env");
    run_app("fdopen");
    run_app("fnmatch");
    run_app("fscanf");
    run_app("fwscanf");
    run_app("iconv_open");
    run_app("inet_pton");
    run_app("mbc");
    run_app("memstream");
    run_app("pthread_cancel_points");
    run_app("pthread_cancel");
    run_app("pthread_cond");
    run_app("pthread_tsd");
    run_app("qsort");
    run_app("random");
    run_app("search_hsearch");
    run_app("search_insque");
    run_app("search_lsearch");
    run_app("search_tsearch");
    run_app("setjmp");
    run_app("snprintf");
    run_app("socket");
    run_app("sscanf");
    run_app("sscanf_long");
    run_app("stat");
    run_app("strftime");
    run_app("string");
    run_app("string_memcpy");
    run_app("string_memmem");
    run_app("string_memset");
    run_app("string_strchr");
    run_app("string_strcspn");
    run_app("string_strstr");
    run_app("strptime");
    run_app("strtod");
    run_app("strtod_simple");
    run_app("strtof");
    run_app("strtol");
    run_app("strtold");
    run_app("swprintf");
    run_app("tgmath");
    run_app("time");
    run_app("tls_align");
    run_app("udiv");
    run_app("ungetc");
    run_app("utime");
    run_app("wcsstr");
    run_app("wcstol");
    run_app("pleval");
    run_app("daemon_failure");
    run_app("dn_expand_empty");
    run_app("dn_expand_ptr_0");
    run_app("fflush_exit");
    run_app("fgets_eof");
    run_app("fgetwc_buffering");
    run_app("flockfile_list");
    run_app("fpclassify_invalid_ld80");
    run_app("ftello_unflushed_append");
    run_app("getpwnam_r_crash");
    run_app("getpwnam_r_errno");
    run_app("iconv_roundtrips");
    run_app("inet_ntop_v4mapped");
    run_app("inet_pton_empty_last_field");
    run_app("iswspace_null");
    run_app("lrand48_signextend");
    run_app("lseek_large");
    run_app("malloc_0");
    run_app("mbsrtowcs_overflow");
    run_app("memmem_oob_read");
    run_app("memmem_oob");
    run_app("mkdtemp_failure");
    run_app("mkstemp_failure");
    run_app("printf_1e9_oob");
    run_app("printf_fmt_g_round");
    run_app("printf_fmt_g_zeros");
    run_app("printf_fmt_n");
    run_app("pthread_robust_detach");
    run_app("pthread_cancel_sem_wait");
    run_app("pthread_cond_smasher");
    run_app("pthread_condattr_setclock");
    run_app("pthread_exit_cancel");
    run_app("pthread_once_deadlock");
    run_app("pthread_rwlock_ebusy");
    run_app("putenv_doublefree");
    run_app("regex_backref_0");
    run_app("regex_bracket_icase");
    run_app("regex_ere_backref");
    run_app("regex_escaped_high_byte");
    run_app("regex_negated_range");
    run_app("regexec_nosub");
    run_app("rewind_clear_error");
    run_app("rlimit_open_files");
    run_app("scanf_bytes_consumed");
    run_app("scanf_match_literal_eof");
    run_app("scanf_nullbyte_char");
    run_app("setvbuf_unget");
    run_app("sigprocmask_internal");
    run_app("sscanf_eof");
    run_app("statvfs");
    run_app("strverscmp");
    run_app("syscall_sign_extend");
    run_app("uselocale_0");
    run_app("wcsncpy_read_overflow");
    run_app("wcsstr_false_negative");
}
