#![no_std]
#![no_main]

mod syscall;
mod runtime;
mod console;

use syscall::*;
use core::slice::from_raw_parts_mut;
use core::str::from_utf8_unchecked;
use core::mem::size_of;
use core::assert;

fn main() {
    let mut envp: [usize; 3] = [0; 3];
    let mut path: [u8; 512] = [0; 512];
    let mut i = 0;
    let mut ch: [u8; 1] = [0];
    print!("bash$ ");
    while syscall_read(0, &mut ch) > 0 {
        if ch[0] == 13 {
            // \n
            path[i] = '\0' as u8;
            let cpid = syscall_fork();
            if cpid > 0 {
                let mut wstatus = 0;
                let mut rusage = 0;
                let childpid = syscall_wait4(-1, &mut wstatus, 0, &mut rusage);
                path = [0; 512];
                i = 0;
                print!("bash$ ");
                continue;
            } else if cpid == 0 {
                let argv = parse_cmd_line(&mut path, i);
                let path = unsafe {from_utf8_unchecked(path.as_slice())};
                syscall_execve(path, &argv, &envp);
                println!("execve error: {}", path);
                return;
            } else {
                return 
            }
        } else if ch[0] == 127 {
            // delete
            if i > 0 {
                i -= 1;
                path[i] = '\0' as u8;
                let path = unsafe {from_utf8_unchecked(path.as_slice())};
                print!("\nbash$ {}", path);
            }
            continue;
        }
        path[i] = ch[0];
        i += 1;
    }
    println!("shell end");
}

fn parse_cmd_line(line: &mut [u8; 512], length: usize) -> [usize; 10] {
    let mut i = 0;
    let mut argv: [usize; 10] = [0; 10];
    argv[0] = line as *const u8 as usize;
    let mut argvi = 1;
    for i in 0..length {
        if line[i] == 32 as u8 {
            // <space>
            if i != length - 1 {
                // argv保持下一个字符指针
                argv[argvi] = (line as *const u8 as usize + i + 1) as usize;
                argvi += 1;
            }
            line[i] = 0 as u8;
        }
     };
    argv

}
